/*
    Title:    UART library
    Author:   Volker Oth
    Date:     5/1999
    Purpose:  Sending a block of data to uart.
    needed
    Software: AVR-GCC to compile, AVA to assemble and link
    needed
    Hardware: ATS90S8515 on STK200 board
    Note:     To contact me, mail to
                  volkeroth@gmx.de
              You might find more AVR related stuff at my homepage:
                  http://members.xoom.com/volkeroth
*/

#include <io.h>
#include <sig-avr.h>
#include <interrupt.h>
#include "timer.h"
#include "uart.h"


/* uart globals */
volatile BYTE uart_txd_buf_cnt;
volatile BYTE uart_rxd_buf_cnt;
volatile BYTE *uart_txd_in_ptr, *uart_txd_out_ptr;
volatile BYTE *uart_rxd_in_ptr, *uart_rxd_out_ptr;
BYTE uart_txd_buffer[UART_OUTBUF_SIZE];
BYTE uart_rxd_buffer[UART_INBUF_SIZE];


void uart_init(void)
/* initialize uart */
{
    /* enable RxD/TxD, receive complete int */
    outp((1<<RXCIE)|(1<<RXEN)|(1<<TXEN),UCR);       
    /* set baud rate */
    outp((BYTE)UART_BAUD_SELECT, UBRR);          
    uart_txd_in_ptr  = uart_txd_out_ptr = uart_txd_buffer;
    uart_rxd_in_ptr  = uart_rxd_out_ptr = uart_rxd_buffer;
    uart_txd_buf_cnt = 0;
    uart_rxd_buf_cnt = 0;
}


SIGNAL(SIG_UART_DATA)      
/* signal handler for uart data buffer empty interrupt */
{
    if (uart_txd_buf_cnt > 0) {
        outp(*uart_txd_out_ptr, UDR);           /* write byte to data buffer */
        if (++uart_txd_out_ptr >= uart_txd_buffer + UART_OUTBUF_SIZE) /* Pointer wrapping */
            uart_txd_out_ptr = uart_txd_buffer;
        if(--uart_txd_buf_cnt == 0)             /* if buffer is empty: */
            cbi(UCR, UDRIE);                    /* disable UDRIE int */
    }
}


SIGNAL(SIG_UART_RECV)      
/* signal handler for receive complete interrupt */
{
    *uart_rxd_in_ptr = inp(UDR);            /* read byte from receive register */
    uart_rxd_buf_cnt++;
    if (++uart_rxd_in_ptr >= uart_rxd_buffer + UART_INBUF_SIZE) /* Pointer wrapping */
        uart_rxd_in_ptr = uart_rxd_buffer;
}


BYTE uart_getchar(void)
{
    BYTE c;

    if (uart_rxd_buf_cnt>0) {
        cli();
        uart_rxd_buf_cnt--;
        c = *uart_rxd_out_ptr;              /* get character from buffer */
        if (++uart_rxd_out_ptr >= uart_rxd_buffer + UART_INBUF_SIZE) /* pointer wrapping */
            uart_rxd_out_ptr = uart_rxd_buffer;
        sei();
        return c;
    } 
    else 
        return 0;                          /* buffer is empty */
}


bool uart_putchar(BYTE c)
{
    if (uart_txd_buf_cnt<UART_OUTBUF_SIZE) {
        cli();
        uart_txd_buf_cnt++;
        *uart_txd_in_ptr = c;               /* put character into buffer */
        if (++uart_txd_in_ptr >= uart_txd_buffer + UART_OUTBUF_SIZE) /* pointer wrapping */
            uart_txd_in_ptr = uart_txd_buffer;
        sbi(UCR, UDRIE);                    /* enable UDRIE int */
        sei();
        return 1;
    } 
    else 
        return 0;                           /* buffer is full */
}

void uart_putstr(BYTE s[])
{
    if (s) {
        char *c = s;

        while (*c)
            if (uart_putchar(*c))
                c++;
    }
}
